# Generated by Django 3.2.4 on 2021-07-30 05:12

from django.db import migrations, models
import xauth.accounts.abstract_models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="User",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                ("last_login", models.DateTimeField(blank=True, null=True, verbose_name="last login")),
                ("email", models.EmailField(db_index=True, max_length=150, unique=True)),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                ("is_staff", models.BooleanField(default=False)),
                ("is_verified", models.BooleanField(default=xauth.accounts.abstract_models.default_is_verified)),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "swappable": "AUTH_USER_MODEL",
            },
        ),
    ]



# Generated by Django 3.2.4 on 2021-07-30 05:12

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="FailedSignInAttempt",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("device_ip", models.GenericIPAddressField(blank=True, db_index=True, null=True, unpack_ipv4=True)),
                ("attempt_date", models.DateField(default=django.utils.timezone.now)),
                ("attempt_count", models.IntegerField(default=1)),
                ("updated_at", models.DateTimeField(auto_now=True)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="PasswordResetLog",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("type", models.CharField(choices=[("Reset", 1), ("Change", 2)], default=2, max_length=10)),
                ("request_ip", models.GenericIPAddressField(blank=True, db_index=True, null=True, unpack_ipv4=True)),
                ("change_ip", models.GenericIPAddressField(blank=True, db_index=True, null=True, unpack_ipv4=True)),
                ("request_time", models.DateTimeField(default=django.utils.timezone.now)),
                ("change_time", models.DateTimeField(null=True)),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from datetime import timedelta
from pathlib import Path

from django.conf import settings
from django.utils.translation import gettext_lazy as _

__all__ = [
    "ENFORCE_ACCOUNT_VERIFICATION",
    "ACCESS_TOKEN_EXPIRY",
    "TOKEN_EXPIRY",
    "ACCOUNT_VERIFICATION_TOKEN_EXPIRY",
    "PASSWORD_RESET_TOKEN_EXPIRY",
    "REPLY_TO_ACCOUNTS_EMAIL_ADDRESSES",
    "ACCOUNT_ACTIVATION_TOKEN_EXPIRY",
    "VERIFY_ENCRYPTED_TOKEN",
    "SENDER_ADDRESS",
    "SENDER_ADDRESS_PASSWORD",
    "APP_NAME",
    "PASSWORD_RESET_REQUEST_SUBJECT",
    "VERIFICATION_REQUEST_SUBJECT",
    "AUTH_APP_LABEL",
    "KEYS_DIR",
    "JWT_SIG_ALG",
    "MAKE_KEY_DIRS",
]

AUTH_APP_LABEL = getattr(settings, "XAUTH_AUTH_APP_LABEL", "accounts")
APP_NAME = getattr(settings, "XAUTH_EMAIL_APP_NAME", "")
PASSWORD_RESET_REQUEST_SUBJECT = getattr(settings, "XAUTH_PASSWORD_RESET_REQUEST_SUBJECT", _("Password Reset Request"))
VERIFICATION_REQUEST_SUBJECT = getattr(settings, "XAUTH_VERIFICATION_REQUEST_SUBJECT", _("Account Verification"))
ENFORCE_ACCOUNT_VERIFICATION = getattr(settings, "XAUTH_ENFORCE_ACCOUNT_VERIFICATION", True)
ACCESS_TOKEN_EXPIRY = TOKEN_EXPIRY = getattr(settings, "XAUTH_ACCESS_TOKEN_EXPIRY", timedelta(days=1))
ACCOUNT_VERIFICATION_TOKEN_EXPIRY = getattr(settings, "XAUTH_ACCOUNT_VERIFICATION_TOKEN_EXPIRY", timedelta(minutes=30))
PASSWORD_RESET_TOKEN_EXPIRY = getattr(settings, "XAUTH_PASSWORD_RESET_TOKEN_EXPIRY", timedelta(minutes=30))
ACCOUNT_ACTIVATION_TOKEN_EXPIRY = getattr(settings, "XAUTH_ACCOUNT_ACTIVATION_TOKEN_EXPIRY", timedelta(minutes=30))
SENDER_ADDRESS = getattr(settings, "XAUTH_SENDER_ADDRESS", settings.EMAIL_HOST_USER)
SENDER_ADDRESS_PASSWORD = getattr(settings, "XAUTH_SENDER_ADDRESS_PASSWORD", settings.EMAIL_HOST_PASSWORD)
REPLY_TO_ACCOUNTS_EMAIL_ADDRESSES = getattr(settings, "XAUTH_REPLY_TO_ACCOUNTS_EMAIL_ADDRESSES", None)
VERIFY_ENCRYPTED_TOKEN = getattr(settings, "XAUTH_VERIFY_ENCRYPTED_TOKEN", True)
# Create a folder in the root directory of the project to hold generated keys
# This directory should not be committed to version control
KEYS_DIR = str(getattr(settings, "XAUTH_KEYS_DIR", Path(settings.BASE_DIR).parent / ".secrets"))
JWT_SIG_ALG = getattr(settings, "XAUTH_JWT_SIG_ALG", "RS256")
MAKE_KEY_DIRS = getattr(settings, "XAUTH_MAKE_KEY_DIRS", True)



from rest_framework.routers import DefaultRouter

__all__ = ["ViewSetBasenameMixin"]


class ViewSetBasenameMixin:
    # See https://www.django-rest-framework.org/api-guide/routers/#usage
    url_name_basename = None

    def __init__(self, *args, **kwargs):
        # This can ease calling `view.reverse_action(...)`; where view is an instance of `ViewSet` created
        # using constructor e.g. `ViewSet(...)`
        kwargs.setdefault("basename", self.__class__._basename())
        super().__init__(*args, **kwargs)

    @classmethod
    def _basename(cls):
        return cls.url_name_basename or DefaultRouter().get_default_basename(cls)



import threading

from django.core.mail import EmailMultiAlternatives, get_connection
from django.template.loader import render_to_string, TemplateDoesNotExist

from xauth.internal_settings import SENDER_ADDRESS, SENDER_ADDRESS_PASSWORD, REPLY_TO_ACCOUNTS_EMAIL_ADDRESSES, APP_NAME

__all__ = ["Mail"]


class Mail:
    def __init__(self, subject=None, recipients=None):
        self._sender = SENDER_ADDRESS
        self._sender_password = SENDER_ADDRESS_PASSWORD
        self._subject = subject
        self._recipients = recipients
        self._reply_to = REPLY_TO_ACCOUNTS_EMAIL_ADDRESSES

    def add_recipient(self, recipient):
        self._recipients = self._recipients or set()
        self._recipients.add(recipient)
        return self

    def send(self, template_name, context=None, request=None, sync=False):
        if context:
            context.setdefault("subject", self._subject)
            context.setdefault("app_name", APP_NAME)

        mail = EmailMultiAlternatives(
            subject=self._subject,
            to=self._recipients,
            from_email=self._sender,
            reply_to=self._reply_to,
            body=render_to_string(f"accounts/{template_name}.txt", context=context, request=request),
            connection=get_connection(username=self._sender, password=self._sender_password),
        )
        try:
            html = render_to_string(f"accounts/{template_name}.html", context=context, request=request)
            mail.attach_alternative(html, "text/html")
        except TemplateDoesNotExist:
            pass

        if sync:
            return mail.send()
        return threading.Thread(target=mail.send).start()



# Generated by Django 3.2.4 on 2021-07-30 05:12

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("xauth", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("accounts", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="SecurityQuestion",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("question", models.CharField(max_length=255, unique=True)),
                ("added_on", models.DateTimeField(auto_now_add=True)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="passwordresetlog",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, related_name="password_reset", to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.AddField(
            model_name="failedsigninattempt",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, related_name="signin_attempts", to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.CreateModel(
            name="Security",
            fields=[
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        related_name="security",
                        serialize=False,
                        to="xauth.user",
                    ),
                ),
                ("security_question_answer", models.CharField(max_length=150, null=True)),
                ("temporary_password", models.CharField(max_length=128, null=True)),
                ("verification_code", models.CharField(max_length=128, null=True)),
                ("temporary_password_generation_time", models.DateTimeField(blank=True, null=True)),
                ("verification_code_generation_time", models.DateTimeField(blank=True, null=True)),
                ("account_deactivation_time", models.DateTimeField(blank=True, null=True)),
                (
                    "security_question",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="accounts.securityquestion",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "unique_together": {("user", "security_question")},
            },
        ),
    ]



from django.contrib.auth.base_user import BaseUserManager

__all__ = ["UserManager"]


class UserManager(BaseUserManager):
    def create_user(self, email, **kwargs):
        if not email:
            raise ValueError("email is required")

        commit = kwargs.pop("save", True)
        password = kwargs.pop("password", None)
        user = self.model(email=self.normalize_email(email), **kwargs)
        user.set_password(password)
        if commit:
            user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **kwargs):
        if not password:
            raise ValueError("superuser password is required")

        user = self.create_user(email, password=password, save=False, **kwargs)
        user.is_staff = True
        user.is_superuser = True
        user.is_verified = True
        user.save(using=self._db)
        return user



from django.conf import settings
from django.urls import include, path
from django.utils.module_loading import import_string
from xently.core.loading import get_classes

from xauth.internal_settings import AUTH_APP_LABEL

(AccountViewSet, SecurityQuestionViewSet) = get_classes(
    f"{AUTH_APP_LABEL}.views", ["AccountViewSet", "SecurityQuestionViewSet"]
)

router = import_string(getattr(settings, "XAUTH_URL_ROUTER", "rest_framework.routers.SimpleRouter"))()
router.register(AUTH_APP_LABEL, AccountViewSet)
router.register("security-questions", SecurityQuestionViewSet)

urlpatterns = [path("", include(router.urls))]
